#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
from telegram import PhotoSize, UserProfilePhotos
from tests.auxil.slots import mro_slots


class UserProfilePhotosTestBase:
    total_count = 2
    photos = [
        [
            PhotoSize("file_id1", "file_un_id1", 512, 512),
            PhotoSize("file_id2", "file_un_id1", 512, 512),
        ],
        [
            PhotoSize("file_id3", "file_un_id3", 512, 512),
            PhotoSize("file_id4", "file_un_id4", 512, 512),
        ],
    ]


class TestUserProfilePhotosWithoutRequest(UserProfilePhotosTestBase):
    def test_slot_behaviour(self):
        inst = UserProfilePhotos(self.total_count, self.photos)
        for attr in inst.__slots__:
            assert getattr(inst, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(inst)) == len(set(mro_slots(inst))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {"total_count": 2, "photos": [[y.to_dict() for y in x] for x in self.photos]}
        user_profile_photos = UserProfilePhotos.de_json(json_dict, offline_bot)
        assert user_profile_photos.api_kwargs == {}
        assert user_profile_photos.total_count == self.total_count
        assert user_profile_photos.photos == tuple(tuple(p) for p in self.photos)

    def test_to_dict(self):
        user_profile_photos = UserProfilePhotos(self.total_count, self.photos)
        user_profile_photos_dict = user_profile_photos.to_dict()
        assert user_profile_photos_dict["total_count"] == user_profile_photos.total_count
        for ix, x in enumerate(user_profile_photos_dict["photos"]):
            for iy, y in enumerate(x):
                assert y == user_profile_photos.photos[ix][iy].to_dict()

    def test_equality(self):
        a = UserProfilePhotos(2, self.photos)
        b = UserProfilePhotos(2, self.photos)
        c = UserProfilePhotos(1, [self.photos[0]])
        d = PhotoSize("file_id1", "unique_id", 512, 512)

        assert a == b
        assert hash(a) == hash(b)

        assert a != c
        assert hash(a) != hash(c)

        assert a != d
        assert hash(a) != hash(d)



#!/usr/bin/env python
#
#  A library that provides a Python interface to the Telegram Bot API
#  Copyright (C) 2015-2026
#  Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser Public License for more details.
#
#  You should have received a copy of the GNU Lesser Public License
#  along with this program.  If not, see [http://www.gnu.org/licenses/].
import datetime as dtm
import re

from telegram import Chat, Message, MessageEntity, Update, User
from tests.auxil.ci_bots import BOT_INFO_PROVIDER
from tests.auxil.pytest_classes import make_bot

CMD_PATTERN = re.compile(r"/[\da-z_]{1,32}(?:@\w{1,32})?")
DATE = dtm.datetime.now()


def make_message(text: str, offline: bool = True, **kwargs):
    """
    Testing utility factory to create a fake ``telegram.Message`` with
    reasonable defaults for mimicking a real message.
    :param text: (str) message text
    :param offline: (bool) whether the bot should be offline
    :return: a (fake) ``telegram.Message``
    """
    bot = kwargs.pop("bot", None)
    if bot is None:
        bot = make_bot(BOT_INFO_PROVIDER.get_info(), offline=offline)
    message = Message(
        message_id=1,
        from_user=kwargs.pop("user", User(id=1, first_name="", is_bot=False)),
        date=kwargs.pop("date", DATE),
        chat=kwargs.pop("chat", Chat(id=1, type="")),
        text=text,
        **kwargs,
    )
    message.set_bot(bot)
    return message


def make_command_message(text, **kwargs):
    """
    Testing utility factory to create a message containing a single telegram
    command.
    Mimics the Telegram API in that it identifies commands within the message
    and tags the returned ``Message`` object with the appropriate ``MessageEntity``
    tag (but it does this only for commands).

    :param text: (str) message text containing (or not) the command
    :return: a (fake) ``telegram.Message`` containing only the command
    """

    match = re.search(CMD_PATTERN, text)
    entities = (
        [
            MessageEntity(
                type=MessageEntity.BOT_COMMAND, offset=match.start(0), length=len(match.group(0))
            )
        ]
        if match
        else []
    )

    return make_message(text, entities=entities, **kwargs)


def make_message_update(message, message_factory=make_message, edited=False, **kwargs):
    """
    Testing utility factory to create an update from a message, as either a
    ``telegram.Message`` or a string. In the latter case ``message_factory``
    is used to convert ``message`` to a ``telegram.Message``.
    :param message: either a ``telegram.Message`` or a string with the message text
    :param message_factory: function to convert the message text into a ``telegram.Message``
    :param edited: whether the message should be stored as ``edited_message`` (vs. ``message``)
    :return: ``telegram.Update`` with the given message
    """
    if not isinstance(message, Message):
        message = message_factory(message, **kwargs)
    update_kwargs = {"message" if not edited else "edited_message": message}
    return Update(0, **update_kwargs)


def make_command_update(message, edited=False, **kwargs):
    """
    Testing utility factory to create an update from a message that potentially
    contains a command. See ``make_command_message`` for more details.
    :param message: message potentially containing a command
    :param edited: whether the message should be stored as ``edited_message`` (vs. ``message``)
    :return: ``telegram.Update`` with the given message
    """
    return make_message_update(message, make_command_message, edited, **kwargs)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import os
from io import BytesIO
from pathlib import Path
from tempfile import TemporaryFile, mkstemp

import pytest

from telegram import File, FileCredentials, Voice
from telegram.error import TelegramError
from tests.auxil.files import data_file
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def file(bot):
    file = File(
        FileTestBase.file_id,
        FileTestBase.file_unique_id,
        file_path=FileTestBase.file_path,
        file_size=FileTestBase.file_size,
    )
    file.set_bot(bot)
    file._unfreeze()
    return file


@pytest.fixture(scope="module")
def encrypted_file(bot):
    # check https://github.com/python-telegram-bot/python-telegram-bot/wiki/\
    # PTB-test-writing-knowledge-base#how-to-generate-encrypted-passport-files
    # if you want to know the source of these values
    fc = FileCredentials(
        "Oq3G4sX+bKZthoyms1YlPqvWou9esb+z0Bi/KqQUG8s=",
        "Pt7fKPgYWKA/7a8E64Ea1X8C+Wf7Ky1tF4ANBl63vl4=",
    )
    ef = File(
        FileTestBase.file_id,
        FileTestBase.file_unique_id,
        FileTestBase.file_size,
        FileTestBase.file_path,
    )
    ef.set_bot(bot)
    ef.set_credentials(fc)
    return ef


@pytest.fixture(scope="module")
def encrypted_local_file(bot):
    # check encrypted_file() for the source of the fc values
    fc = FileCredentials(
        "Oq3G4sX+bKZthoyms1YlPqvWou9esb+z0Bi/KqQUG8s=",
        "Pt7fKPgYWKA/7a8E64Ea1X8C+Wf7Ky1tF4ANBl63vl4=",
    )
    ef = File(
        FileTestBase.file_id,
        FileTestBase.file_unique_id,
        FileTestBase.file_size,
        file_path=str(data_file("image_encrypted.jpg")),
    )
    ef.set_bot(bot)
    ef.set_credentials(fc)
    return ef


@pytest.fixture(scope="module")
def local_file(bot):
    file = File(
        FileTestBase.file_id,
        FileTestBase.file_unique_id,
        file_path=str(data_file("local_file.txt")),
        file_size=FileTestBase.file_size,
    )
    file.set_bot(bot)
    return file


class FileTestBase:
    file_id = "NOTVALIDDOESNOTMATTER"
    file_unique_id = "adc3145fd2e84d95b64d68eaa22aa33e"
    file_path = (
        "https://api.org/file/bot133505823:AAHZFMHno3mzVLErU5b5jJvaeG--qUyLyG0/document/file_3"
    )
    file_size = 28232
    file_content = "Saint-Saëns".encode()  # Intentionally contains unicode chars.


class TestFileWithoutRequest(FileTestBase):
    def test_slot_behaviour(self, file):
        for attr in file.__slots__:
            assert getattr(file, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(file)) == len(set(mro_slots(file))), "duplicate slot"

    def test_de_json(self, offline_bot):
        json_dict = {
            "file_id": self.file_id,
            "file_unique_id": self.file_unique_id,
            "file_path": self.file_path,
            "file_size": self.file_size,
        }
        new_file = File.de_json(json_dict, offline_bot)
        assert new_file.api_kwargs == {}

        assert new_file.file_id == self.file_id
        assert new_file.file_unique_id == self.file_unique_id
        assert new_file.file_path == self.file_path
        assert new_file.file_size == self.file_size

    def test_to_dict(self, file):
        file_dict = file.to_dict()

        assert isinstance(file_dict, dict)
        assert file_dict["file_id"] == file.file_id
        assert file_dict["file_unique_id"] == file.file_unique_id
        assert file_dict["file_path"] == file.file_path
        assert file_dict["file_size"] == file.file_size

    def test_equality(self, offline_bot):
        a = File(self.file_id, self.file_unique_id, offline_bot)
        b = File("", self.file_unique_id, offline_bot)
        c = File(self.file_id, self.file_unique_id, None)
        d = File("", "", offline_bot)
        e = Voice(self.file_id, self.file_unique_id, 0)

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)

    async def test_download(self, monkeypatch, file):
        async def test(*args, **kwargs):
            return self.file_content

        monkeypatch.setattr(file.get_bot().request, "retrieve", test)
        out_file = await file.download_to_drive()

        try:
            assert out_file.read_bytes() == self.file_content
        finally:
            out_file.unlink(missing_ok=True)

    @pytest.mark.parametrize(
        "custom_path_type", [str, Path], ids=["str custom_path", "pathlib.Path custom_path"]
    )
    async def test_download_custom_path(self, monkeypatch, file, custom_path_type):
        async def test(*args, **kwargs):
            return self.file_content

        monkeypatch.setattr(file.get_bot().request, "retrieve", test)
        file_handle, custom_path = mkstemp()
        custom_path = Path(custom_path)
        try:
            out_file = await file.download_to_drive(custom_path_type(custom_path))
            assert out_file == custom_path
            assert out_file.read_bytes() == self.file_content
        finally:
            os.close(file_handle)
            custom_path.unlink(missing_ok=True)

    async def test_download_file_obj(self, monkeypatch, file):
        async def test(*args, **kwargs):
            return self.file_content

        monkeypatch.setattr(file.get_bot().request, "retrieve", test)
        with TemporaryFile() as custom_fobj:
            await file.download_to_memory(out=custom_fobj)
            custom_fobj.seek(0)
            assert custom_fobj.read() == self.file_content

    async def test_download_bytearray(self, monkeypatch, file):
        async def test(*args, **kwargs):
            return self.file_content

        monkeypatch.setattr(file.get_bot().request, "retrieve", test)

        # Check that a download to a newly allocated bytearray works.
        buf = await file.download_as_bytearray()
        assert buf == bytearray(self.file_content)

        # Check that a download to a given bytearray works (extends the bytearray).
        buf2 = buf[:]
        buf3 = await file.download_as_bytearray(buf=buf2)
        assert buf3 is buf2
        assert buf2[len(buf) :] == buf
        assert buf2[: len(buf)] == buf

    async def test_download_encrypted(self, monkeypatch, offline_bot, encrypted_file):
        async def test(*args, **kwargs):
            return data_file("image_encrypted.jpg").read_bytes()

        monkeypatch.setattr(encrypted_file.get_bot().request, "retrieve", test)
        out_file = await encrypted_file.download_to_drive()

        try:
            assert out_file.read_bytes() == data_file("image_decrypted.jpg").read_bytes()
        finally:
            out_file.unlink(missing_ok=True)

    async def test_download_file_obj_encrypted(self, monkeypatch, encrypted_file):
        async def test(*args, **kwargs):
            return data_file("image_encrypted.jpg").read_bytes()

        monkeypatch.setattr(encrypted_file.get_bot().request, "retrieve", test)
        with TemporaryFile() as custom_fobj:
            await encrypted_file.download_to_memory(out=custom_fobj)
            custom_fobj.seek(0)
            assert custom_fobj.read() == data_file("image_decrypted.jpg").read_bytes()

    async def test_download_file_obj_local_file_encrypted(self, monkeypatch, encrypted_local_file):
        async def test(*args, **kwargs):
            return data_file("image_encrypted.jpg").read_bytes()

        monkeypatch.setattr(encrypted_local_file.get_bot().request, "retrieve", test)
        with TemporaryFile() as custom_fobj:
            await encrypted_local_file.download_to_memory(out=custom_fobj)
            custom_fobj.seek(0)
            assert custom_fobj.read() == data_file("image_decrypted.jpg").read_bytes()

    async def test_download_bytearray_encrypted(self, monkeypatch, encrypted_file):
        async def test(*args, **kwargs):
            return data_file("image_encrypted.jpg").read_bytes()

        monkeypatch.setattr(encrypted_file.get_bot().request, "retrieve", test)

        # Check that a download to a newly allocated bytearray works.
        buf = await encrypted_file.download_as_bytearray()
        assert buf == bytearray(data_file("image_decrypted.jpg").read_bytes())

        # Check that a download to a given bytearray works (extends the bytearray).
        buf2 = buf[:]
        buf3 = await encrypted_file.download_as_bytearray(buf=buf2)
        assert buf3 is buf2
        assert buf2[len(buf) :] == buf
        assert buf2[: len(buf)] == buf

    async def test_download_no_file_path(self):
        with pytest.raises(RuntimeError, match="No `file_path` available"):
            await File(self.file_id, self.file_unique_id).download_to_drive()
        with pytest.raises(RuntimeError, match="No `file_path` available"):
            await File(self.file_id, self.file_unique_id).download_to_memory(BytesIO())
        with pytest.raises(RuntimeError, match="No `file_path` available"):
            await File(self.file_id, self.file_unique_id).download_as_bytearray()


class TestFileWithRequest(FileTestBase):
    async def test_error_get_empty_file_id(self, bot):
        with pytest.raises(TelegramError):
            await bot.get_file(file_id="")

    async def test_download_local_file(self, local_file):
        assert await local_file.download_to_drive() == Path(local_file.file_path)
        # Ensure that the file contents didn't change
        assert Path(local_file.file_path).read_bytes() == self.file_content

    @pytest.mark.parametrize(
        "custom_path_type", [str, Path], ids=["str custom_path", "pathlib.Path custom_path"]
    )
    async def test_download_custom_path_local_file(self, local_file, custom_path_type):
        file_handle, custom_path = mkstemp()
        custom_path = Path(custom_path)
        try:
            out_file = await local_file.download_to_drive(custom_path_type(custom_path))
            assert out_file == custom_path
            assert out_file.read_bytes() == self.file_content
        finally:
            os.close(file_handle)
            custom_path.unlink(missing_ok=True)

    async def test_download_file_obj_local_file(self, local_file):
        with TemporaryFile() as custom_fobj:
            await local_file.download_to_memory(out=custom_fobj)
            custom_fobj.seek(0)
            assert custom_fobj.read() == self.file_content

    @pytest.mark.parametrize(
        "custom_path_type", [str, Path], ids=["str custom_path", "pathlib.Path custom_path"]
    )
    async def test_download_custom_path_local_file_encrypted(
        self, encrypted_local_file, custom_path_type
    ):
        file_handle, custom_path = mkstemp()
        custom_path = Path(custom_path)
        try:
            out_file = await encrypted_local_file.download_to_drive(custom_path_type(custom_path))
            assert out_file == custom_path
            assert out_file.read_bytes() == data_file("image_decrypted.jpg").read_bytes()
        finally:
            os.close(file_handle)
            custom_path.unlink(missing_ok=True)

    async def test_download_local_file_encrypted(self, encrypted_local_file):
        out_file = await encrypted_local_file.download_to_drive()
        try:
            assert out_file.read_bytes() == data_file("image_decrypted.jpg").read_bytes()
        finally:
            out_file.unlink(missing_ok=True)

    async def test_download_bytearray_local_file(self, local_file):
        # Check that a download to a newly allocated bytearray works.
        buf = await local_file.download_as_bytearray()
        assert buf == bytearray(self.file_content)

        # Check that a download to a given bytearray works (extends the bytearray).
        buf2 = buf[:]
        buf3 = await local_file.download_as_bytearray(buf=buf2)
        assert buf3 is buf2
        assert buf2[len(buf) :] == buf
        assert buf2[: len(buf)] == buf

    async def test_download_bytearray_local_file_encrypted(self, encrypted_local_file):
        # Check that a download to a newly allocated bytearray works.
        buf = await encrypted_local_file.download_as_bytearray()
        assert buf == bytearray(data_file("image_decrypted.jpg").read_bytes())

        # Check that a download to a given bytearray works (extends the bytearray).
        buf2 = buf[:]
        buf3 = await encrypted_local_file.download_as_bytearray(buf=buf2)
        assert buf3 is buf2
        assert buf2[len(buf) :] == buf
        assert buf2[: len(buf)] == buf



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains an object that represents a Telegram Web App Info."""

from telegram._telegramobject import TelegramObject
from telegram._utils.types import JSONDict


class WebAppInfo(TelegramObject):
    """
    This object contains information about a `Web App <https://core.telegram.org/bots/webapps>`_.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`url` are equal.

    Examples:
        :any:`Webapp Bot <examples.webappbot>`

    .. versionadded:: 20.0

    Args:
        url (:obj:`str`): An HTTPS URL of a Web App to be opened with additional data as specified
            in `Initializing Web Apps \
            <https://core.telegram.org/bots/webapps#initializing-mini-apps>`_.

    Attributes:
        url (:obj:`str`): An HTTPS URL of a Web App to be opened with additional data as specified
            in `Initializing Web Apps \
            <https://core.telegram.org/bots/webapps#initializing-mini-apps>`_.
    """

    __slots__ = ("url",)

    def __init__(self, url: str, *, api_kwargs: JSONDict | None = None):
        super().__init__(api_kwargs=api_kwargs)
        # Required
        self.url: str = url

        self._id_attrs = (self.url,)

        self._freeze()



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
# pylint: disable=redefined-builtin
"""This module contains an object that represents a Telegram CallbackQuery"""

from collections.abc import Sequence
from typing import TYPE_CHECKING, Final

from telegram import constants
from telegram._inputchecklist import InputChecklist
from telegram._message import MaybeInaccessibleMessage, Message
from telegram._telegramobject import TelegramObject
from telegram._user import User
from telegram._utils.argumentparsing import de_json_optional
from telegram._utils.defaultvalue import DEFAULT_NONE
from telegram._utils.types import JSONDict, ODVInput, TimePeriod

if TYPE_CHECKING:
    from telegram import (
        Bot,
        GameHighScore,
        InlineKeyboardMarkup,
        InputMedia,
        LinkPreviewOptions,
        MessageEntity,
        MessageId,
        ReplyParameters,
        SuggestedPostParameters,
    )
    from telegram._files.location import Location
    from telegram._utils.types import ReplyMarkup


class CallbackQuery(TelegramObject):
    """
    This object represents an incoming callback query from a callback button in an inline keyboard.

    If the button that originated the query was attached to a message sent by the bot, the field
    :attr:`message` will be present. If the button was attached to a message sent via the bot (in
    inline mode), the field :attr:`inline_message_id` will be present.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`id` is equal.

    Note:
        * In Python :keyword:`from` is a reserved word. Use :paramref:`from_user` instead.
        * Exactly one of the fields :attr:`data` or :attr:`game_short_name` will be present.
        * After the user presses an inline button, Telegram clients will display a progress bar
          until you call :attr:`answer`. It is, therefore, necessary to react
          by calling :attr:`telegram.Bot.answer_callback_query` even if no notification to the user
          is needed (e.g., without specifying any of the optional parameters).
        * If you're using :attr:`telegram.ext.ExtBot.callback_data_cache`, :attr:`data` may be
          an instance
          of :class:`telegram.ext.InvalidCallbackData`. This will be the case, if the data
          associated with the button triggering the :class:`telegram.CallbackQuery` was already
          deleted or if :attr:`data` was manipulated by a malicious client.

          .. versionadded:: 13.6

    Args:
        id (:obj:`str`): Unique identifier for this query.
        from_user (:class:`telegram.User`): Sender.
        chat_instance (:obj:`str`): Global identifier, uniquely corresponding to the chat to which
            the message with the callback button was sent. Useful for high scores in games.
        message (:class:`telegram.MaybeInaccessibleMessage`, optional): Message sent by the bot
            with the callback button that originated the query.

            .. versionchanged:: 20.8
               Accept objects of type :class:`telegram.MaybeInaccessibleMessage` since Bot API 7.0.
        data (:obj:`str`, optional): Data associated with the callback button. Be aware that the
            message, which originated the query, can contain no callback buttons with this data.
        inline_message_id (:obj:`str`, optional): Identifier of the message sent via the bot in
            inline mode, that originated the query.
        game_short_name (:obj:`str`, optional): Short name of a Game to be returned, serves as
            the unique identifier for the game.

    Attributes:
        id (:obj:`str`): Unique identifier for this query.
        from_user (:class:`telegram.User`): Sender.
        chat_instance (:obj:`str`): Global identifier, uniquely corresponding to the chat to which
            the message with the callback button was sent. Useful for high scores in games.
        message (:class:`telegram.MaybeInaccessibleMessage`): Optional. Message sent by the bot
            with the callback button that originated the query.

            .. versionchanged:: 20.8
               Objects may be of type :class:`telegram.MaybeInaccessibleMessage` since Bot API
               7.0.
        data (:obj:`str` | :obj:`object`): Optional. Data associated with the callback button.
            Be aware that the message, which originated the query, can contain no callback buttons
            with this data.

            Tip:
                The value here is the same as the value passed in
                :paramref:`telegram.InlineKeyboardButton.callback_data`.
        inline_message_id (:obj:`str`): Optional. Identifier of the message sent via the bot in
            inline mode, that originated the query.
        game_short_name (:obj:`str`): Optional. Short name of a Game to be returned, serves as
            the unique identifier for the game.


    """

    __slots__ = (
        "chat_instance",
        "data",
        "from_user",
        "game_short_name",
        "id",
        "inline_message_id",
        "message",
    )

    def __init__(
        self,
        id: str,
        from_user: User,
        chat_instance: str,
        message: MaybeInaccessibleMessage | None = None,
        data: str | None = None,
        inline_message_id: str | None = None,
        game_short_name: str | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)
        # Required
        self.id: str = id
        self.from_user: User = from_user
        self.chat_instance: str = chat_instance
        # Optionals
        self.message: MaybeInaccessibleMessage | None = message
        self.data: str | None = data
        self.inline_message_id: str | None = inline_message_id
        self.game_short_name: str | None = game_short_name

        self._id_attrs = (self.id,)

        self._freeze()

    @classmethod
    def de_json(cls, data: JSONDict, bot: "Bot | None" = None) -> "CallbackQuery":
        """See :meth:`telegram.TelegramObject.de_json`."""
        data = cls._parse_data(data)

        data["from_user"] = de_json_optional(data.pop("from", None), User, bot)
        data["message"] = de_json_optional(data.get("message"), Message, bot)

        return super().de_json(data=data, bot=bot)

    async def answer(
        self,
        text: str | None = None,
        show_alert: bool | None = None,
        url: str | None = None,
        cache_time: TimePeriod | None = None,
        *,
        read_timeout: ODVInput[float] = DEFAULT_NONE,
        write_timeout: ODVInput[float] = DEFAULT_NONE,
        connect_timeout: ODVInput[float] = DEFAULT_NONE,
        pool_timeout: ODVInput[float] = DEFAULT_NONE,
        api_kwargs: JSONDict | None = None,
    ) -> bool:
        """Shortcut for::

             await bot.answer_callback_query(update.callback_query.id, *args, **kwargs)

        For the documentation of the arguments, please see
        :meth:`telegram.Bot.answer_callback_query`.

        Returns:
            :obj:`bool`: On success, :obj:`True` is returned.

        """
        return await self.get_bot().answer_callback_query(
            callback_query_id=self.id,
            text=text,
            show_alert=show_alert,
            url=url,
            cache_time=cache_time,
            read_timeout=read_timeout,
            write_timeout=write_timeout,
            connect_timeout=connect_timeout,
            pool_timeout=pool_timeout,
            api_kwargs=api_kwargs,
        )

    def _get_message(self, action: str = "edit") -> Message:
        """Helper method to get the message for the shortcut methods. Must be called only
        if :attr:`inline_message_id` is *not* set.
        """
        if not isinstance(self.message, Message):
            raise TypeError(f"Cannot {action} an inaccessible message")
        return self.message

    async def edit_message_text(
        self,
        text: str,
        parse_mode: ODVInput[str] = DEFAULT_NONE,
        reply_markup: "InlineKeyboardMarkup | None" = None,
        entities: Sequence["MessageEntity"] | None = None,
        link_preview_options: ODVInput["LinkPreviewOptions"] = DEFAULT_NONE,
        *,
        disable_web_page_preview: bool | None = None,
        read_timeout: ODVInput[float] = DEFAULT_NONE,
        write_timeout: ODVInput[float] = DEFAULT_NONE,
        connect_timeout: ODVInput[float] = DEFAULT_NONE,
        pool_timeout: ODVInput[float] = DEFAULT_NONE,
        api_kwargs: JSONDict | None = None,
    ) -> "Message | bool":
        """Shortcut for either::

            await update.callback_query.message.edit_text(*args, **kwargs)

        or::

            await bot.edit_message_text(
                inline_message_id=update.callback_query.inline_message_id, *args, **kwargs,
            )

        For the documentation of the arguments, please see
        :meth:`telegram.Bot.edit_message_text` and :meth:`telegram.Message.edit_text`.

        .. versionchanged:: 20.8
           Raises :exc:`TypeError` if :attr:`message` is not accessible.

        Returns:
            :class:`telegram.Message`: On success, if edited message is sent by the bot, the
            edited Message is returned, otherwise :obj:`True` is returned.

        Raises:
            :exc:`TypeError` if :attr:`message` is not accessible.

        """
        if self.inline_message_id:
            return await self.get_bot().edit_message_text(
                inline_message_id=self.inline_message_id,
                text=text,
                parse_mode=parse_mode,
                disable_web_page_preview=disable_web_page_preview,
                link_preview_options=link_preview_options,
                reply_markup=reply_markup,
                read_timeout=read_timeout,
                write_timeout=write_timeout,
                connect_timeout=connect_timeout,
                pool_timeout=pool_timeout,
                api_kwargs=api_kwargs,
                entities=entities,
                chat_id=None,
                message_id=None,
                # inline messages can not be sent on behalf of a bcid
                business_connection_id=None,
            )
        return await self._get_message().edit_text(
            text=text,
            parse_mode=parse_mode,
            disable_web_page_preview=disable_web_page_preview,
            link_preview_options=link_preview_options,
            reply_markup=reply_markup,
            read_timeout=read_timeout,
            write_timeout=write_timeout,
            connect_timeout=connect_timeout,
            pool_timeout=pool_timeout,
            api_kwargs=api_kwargs,
            entities=entities,
        )

    async def edit_message_caption(
        self,
        caption: str | None = None,
        reply_markup: "InlineKeyboardMarkup | None" = None,
        parse_mode: ODVInput[str] = DEFAULT_NONE,
        caption_entities: Sequence["MessageEntity"] | None = None,
        show_caption_above_media: bool | None = None,
        *,
        read_timeout: ODVInput[float] = DEFAULT_NONE,
        write_timeout: ODVInput[float] = DEFAULT_NONE,
        connect_timeout: ODVInput[float] = DEFAULT_NONE,
        pool_timeout: ODVInput[float] = DEFAULT_NONE,
        api_kwargs: JSONDict | None = None,
    ) -> "Message | bool":
        """Shortcut for either::

            await update.callback_query.message.edit_caption(*args, **kwargs)

        or::

            await bot.edit_message_caption(
                inline_message_id=update.callback_query.inline_message_id, *args, **kwargs,
            )

        For the documentation of the arguments, please see
        :meth:`telegram.Bot.edit_message_caption` and :meth:`telegram.Message.edit_caption`.

        .. versionchanged:: 20.8
           Raises :exc:`TypeError` if :attr:`message` is not accessible.

        Returns:
            :class:`telegram.Message`: On success, if edited message is sent by the bot, the
            edited Message is returned, otherwise :obj:`True` is returned.

        Raises:
            :exc:`TypeError` if :attr:`message` is not accessible.

        """
        if self.inline_message_id:
            return await self.get_bot().edit_message_caption(
                caption=caption,
                inline_message_id=self.inline_message_id,
                reply_markup=reply_markup,
                read_timeout=read_timeout,
                write_timeout=write_timeout,
                connect_timeout=connect_timeout,
                pool_timeout=pool_timeout,
                parse_mode=parse_mode,
                api_kwargs=api_kwargs,
                caption_entities=caption_entities,
                chat_id=None,
                message_id=None,
                show_caption_above_media=show_caption_above_media,
                # inline messages can not be sent on behalf of a bcid
                business_connection_id=None,
            )
        return await self._get_message().edit_caption(
            caption=caption,
            reply_markup=reply_markup,
            read_timeout=read_timeout,
            write_timeout=write_timeout,
            connect_timeout=connect_timeout,
            pool_timeout=pool_timeout,
            parse_mode=parse_mode,
            api_kwargs=api_kwargs,
            caption_entities=caption_entities,
            show_caption_above_media=show_caption_above_media,
        )

    async def edit_message_checklist(
        self,
        checklist: InputChecklist,
        reply_markup: "InlineKeyboardMarkup | None" = None,
        *,
        read_timeout: ODVInput[float] = DEFAULT_NONE,
        write_timeout: ODVInput[float] = DEFAULT_NONE,
        connect_timeout: ODVInput[float] = DEFAULT_NONE,
        pool_timeout: ODVInput[float] = DEFAULT_NONE,
        api_kwargs: JSONDict | None = None,
    ) -> "Message | bool":
        """Shortcut for::

            await update.callback_query.message.edit_checklist(*args, **kwargs)

        For the documentation of the arguments, please see
        :meth:`telegram.Message.edit_checklist`.

        .. versionadded:: 22.3

        Returns:
            :class:`telegram.Message`: On success, the edited Message is returned.

        Raises:
            :exc:`TypeError` if :attr:`message` is not accessible.

        """
        return await self._get_message().edit_checklist(
            checklist=checklist,
            reply_markup=reply_markup,
            read_timeout=read_timeout,
            write_timeout=write_timeout,
            connect_timeout=connect_timeout,
            pool_timeout=pool_timeout,
            api_kwargs=api_kwargs,
        )

    async def edit_message_reply_markup(
        self,
        reply_markup: "InlineKeyboardMarkup | None" = None,
        *,
        read_timeout: ODVInput[float] = DEFAULT_NONE,
        write_timeout: ODVInput[float] = DEFAULT_NONE,
        connect_timeout: ODVInput[float] = DEFAULT_NONE,
        pool_timeout: ODVInput[float] = DEFAULT_NONE,
        api_kwargs: JSONDict | None = None,
    ) -> "Message | bool":
        """Shortcut for either::

            await update.callback_query.message.edit_reply_markup(*args, **kwargs)

        or::

            await bot.edit_message_reply_markup(
                inline_message_id=update.callback_query.inline_message_id, *args, **kwargs
            )

        For the documentation of the arguments, please see
        :meth:`telegram.Bot.edit_message_reply_markup` and
        :meth:`telegram.Message.edit_reply_markup`.

        .. versionchanged:: 20.8
           Raises :exc:`TypeError` if :attr:`message` is not accessible.

        Returns:
            :class:`telegram.Message`: On success, if edited message is sent by the bot, the
            edited Message is returned, otherwise :obj:`True` is returned.

        Raises:
            :exc:`TypeError` if :attr:`message` is not accessible.

        """
        if self.inline_message_id:
            return await self.get_bot().edit_message_reply_markup(
                reply_markup=reply_markup,
                inline_message_id=self.inline_message_id,
                read_timeout=read_timeout,
                write_timeout=write_timeout,
                connect_timeout=connect_timeout,
                pool_timeout=pool_timeout,
                api_kwargs=api_kwargs,
                chat_id=None,
                message_id=None,
                # inline messages can not be sent on behalf of a bcid
                business_connection_id=None,
            )
        return await self._get_message().edit_reply_markup(
            reply_markup=reply_markup,
            read_timeout=read_timeout,
            write_timeout=write_timeout,
            connect_timeout=connect_timeout,
            pool_timeout=pool_timeout,
            api_kwargs=api_kwargs,
        )

    async def edit_message_media(
        self,
        media: "InputMedia",
        reply_markup: "InlineKeyboardMarkup | None" = None,
        *,
        read_timeout: ODVInput[float] = DEFAULT_NONE,
        write_timeout: ODVInput[float] = DEFAULT_NONE,
        connect_timeout: ODVInput[float] = DEFAULT_NONE,
        pool_timeout: ODVInput[float] = DEFAULT_NONE,
        api_kwargs: JSONDict | None = None,
    ) -> "Message | bool":
        """Shortcut for either::

            await update.callback_query.message.edit_media(*args, **kwargs)

        or::

            await bot.edit_message_media(
                inline_message_id=update.callback_query.inline_message_id, *args, **kwargs
            )

        For the documentation of the arguments, please see
        :meth:`telegram.Bot.edit_message_media` and :meth:`telegram.Message.edit_media`.

        .. versionchanged:: 20.8
           Raises :exc:`TypeError` if :attr:`message` is not accessible.

        Returns:
            :class:`telegram.Message`: On success, if edited message is not an inline message, the
            edited Message is returned, otherwise :obj:`True` is returned.

        Raises:
            :exc:`TypeError` if :attr:`message` is not accessible.

        """
        if self.inline_message_id:
            return await self.get_bot().edit_message_media(
                inline_message_id=self.inline_message_id,
                media=media,
                reply_markup=reply_markup,
                read_timeout=read_timeout,
                write_timeout=write_timeout,
                connect_timeout=connect_timeout,
                pool_timeout=pool_timeout,
                api_kwargs=api_kwargs,
                chat_id=None,
                message_id=None,
                # inline messages can not be sent on behalf of a bcid
                business_connection_id=None,
            )
        return await self._get_message().edit_media(
            media=media,
            reply_markup=reply_markup,
            read_timeout=read_timeout,
            write_timeout=write_timeout,
            connect_timeout=connect_timeout,
            pool_timeout=pool_timeout,
            api_kwargs=api_kwargs,
        )

    async def edit_message_live_location(
        self,
        latitude: float | None = None,
        longitude: float | None = None,
        reply_markup: "InlineKeyboardMarkup | None" = None,
        horizontal_accuracy: float | None = None,
        heading: int | None = None,
        proximity_alert_radius: int | None = None,
        live_period: TimePeriod | None = None,
        *,
        location: "Location | None" = None,
        read_timeout: ODVInput[float] = DEFAULT_NONE,
        write_timeout: ODVInput[float] = DEFAULT_NONE,
        connect_timeout: ODVInput[float] = DEFAULT_NONE,
        pool_timeout: ODVInput[float] = DEFAULT_NONE,
        api_kwargs: JSONDict | None = None,
    ) -> "Message | bool":
        """Shortcut for either::

            await update.callback_query.message.edit_live_location(*args, **kwargs)

        or::

            await bot.edit_message_live_location(
                inline_message_id=update.callback_query.inline_message_id, *args, **kwargs
            )

        For the documentation of the arguments, please see
        :meth:`telegram.Bot.edit_message_live_location` and
        :meth:`telegram.Message.edit_live_location`.

        .. versionchanged:: 20.8
           Raises :exc:`TypeError` if :attr:`message` is not accessible.

        Returns:
            :class:`telegram.Message`: On success, if edited message is sent by the bot, the
            edited Message is returned, otherwise :obj:`True` is returned.

        Raises:
            :exc:`TypeError` if :attr:`message` is not accessible.

        """
        if self.inline_message_id:
            return await self.get_bot().edit_message_live_location(
                inline_message_id=self.inline_message_id,
                latitude=latitude,
                longitude=longitude,
                location=location,
                reply_markup=reply_markup,
                read_timeout=read_timeout,
                write_timeout=write_timeout,
                connect_timeout=connect_timeout,
                pool_timeout=pool_timeout,
                api_kwargs=api_kwargs,
                horizontal_accuracy=horizontal_accuracy,
                heading=heading,
                proximity_alert_radius=proximity_alert_radius,
                live_period=live_period,
                chat_id=None,
                message_id=None,
                # inline messages can not be sent on behalf of a bcid
                business_connection_id=None,
            )
        return await self._get_message().edit_live_location(
            latitude=latitude,
            longitude=longitude,
            location=location,
            reply_markup=reply_markup,
            read_timeout=read_timeout,
            write_timeout=write_timeout,
            connect_timeout=connect_timeout,
            pool_timeout=pool_timeout,
            api_kwargs=api_kwargs,
            horizontal_accuracy=horizontal_accuracy,
            heading=heading,
            proximity_alert_radius=proximity_alert_radius,
            live_period=live_period,
        )

    async def stop_message_live_location(
        self,
        reply_markup: "InlineKeyboardMarkup | None" = None,
        *,
        read_timeout: ODVInput[float] = DEFAULT_NONE,
        write_timeout: ODVInput[float] = DEFAULT_NONE,
        connect_timeout: ODVInput[float] = DEFAULT_NONE,
        pool_timeout: ODVInput[float] = DEFAULT_NONE,
        api_kwargs: JSONDict | None = None,
    ) -> "Message | bool":
        """Shortcut for either::

            await update.callback_query.message.stop_live_location(*args, **kwargs)

        or::

            await bot.stop_message_live_location(
                inline_message_id=update.callback_query.inline_message_id, *args, **kwargs
            )

        For the documentation of the arguments, please see
        :meth:`telegram.Bot.stop_message_live_location` and
        :meth:`telegram.Message.stop_live_location`.

        .. versionchanged:: 20.8
           Raises :exc:`TypeError` if :attr:`message` is not accessible.

        Returns:
            :class:`telegram.Message`: On success, if edited message is sent by the bot, the
            edited Message is returned, otherwise :obj:`True` is returned.

        Raises:
            :exc:`TypeError` if :attr:`message` is not accessible.

        """
        if self.inline_message_id:
            return await self.get_bot().stop_message_live_location(
                inline_message_id=self.inline_message_id,
                reply_markup=reply_markup,
                read_timeout=read_timeout,
                write_timeout=write_timeout,
                connect_timeout=connect_timeout,
                pool_timeout=pool_timeout,
                api_kwargs=api_kwargs,
                chat_id=None,
                message_id=None,
                # inline messages can not be sent on behalf of a bcid
                business_connection_id=None,
            )
        return await self._get_message().stop_live_location(
            reply_markup=reply_markup,
            read_timeout=read_timeout,
            write_timeout=write_timeout,
            connect_timeout=connect_timeout,
            pool_timeout=pool_timeout,
            api_kwargs=api_kwargs,
        )

    async def set_game_score(
        self,
        user_id: int,
        score: int,
        force: bool | None = None,
        disable_edit_message: bool | None = None,
        *,
        read_timeout: ODVInput[float] = DEFAULT_NONE,
        write_timeout: ODVInput[float] = DEFAULT_NONE,
        connect_timeout: ODVInput[float] = DEFAULT_NONE,
        pool_timeout: ODVInput[float] = DEFAULT_NONE,
        api_kwargs: JSONDict | None = None,
    ) -> "Message | bool":
        """Shortcut for either::

           await update.callback_query.message.set_game_score(*args, **kwargs)

        or::

            await bot.set_game_score(
                inline_message_id=update.callback_query.inline_message_id, *args, **kwargs
            )

        For the documentation of the arguments, please see
        :meth:`telegram.Bot.set_game_score` and :meth:`telegram.Message.set_game_score`.

        .. versionchanged:: 20.8
           Raises :exc:`TypeError` if :attr:`message` is not accessible.

        Returns:
            :class:`telegram.Message`: On success, if edited message is sent by the bot, the
            edited Message is returned, otherwise :obj:`True` is returned.

        Raises:
            :exc:`TypeError` if :attr:`message` is not accessible.

        """
        if self.inline_message_id:
            return await self.get_bot().set_game_score(
                inline_message_id=self.inline_message_id,
                user_id=user_id,
                score=score,
                force=force,
                disable_edit_message=disable_edit_message,
                read_timeout=read_timeout,
                write_timeout=write_timeout,
                connect_timeout=connect_timeout,
                pool_timeout=pool_timeout,
                api_kwargs=api_kwargs,
                chat_id=None,
                message_id=None,
            )
        return await self._get_message().set_game_score(
            user_id=user_id,
            score=score,
            force=force,
            disable_edit_message=disable_edit_message,
            read_timeout=read_timeout,
            write_timeout=write_timeout,
            connect_timeout=connect_timeout,
            pool_timeout=pool_timeout,
            api_kwargs=api_kwargs,
        )

    async def get_game_high_scores(
        self,
        user_id: int,
        *,
        read_timeout: ODVInput[float] = DEFAULT_NONE,
        write_timeout: ODVInput[float] = DEFAULT_NONE,
        connect_timeout: ODVInput[float] = DEFAULT_NONE,
        pool_timeout: ODVInput[float] = DEFAULT_NONE,
        api_kwargs: JSONDict | None = None,
    ) -> tuple["GameHighScore", ...]:
        """Shortcut for either::

            await update.callback_query.message.get_game_high_score(*args, **kwargs)

        or::

            await bot.get_game_high_scores(
                inline_message_id=update.callback_query.inline_message_id, *args, **kwargs
            )

        For the documentation of the arguments, please see
        :meth:`telegram.Bot.get_game_high_scores` and
        :meth:`telegram.Message.get_game_high_scores`.

        .. versionchanged:: 20.8
           Raises :exc:`TypeError` if :attr:`message` is not accessible.

        Returns:
            tuple[:class:`telegram.GameHighScore`]

        Raises:
            :exc:`TypeError` if :attr:`message` is not accessible.

        """
        if self.inline_message_id:
            return await self.get_bot().get_game_high_scores(
                inline_message_id=self.inline_message_id,
                user_id=user_id,
                read_timeout=read_timeout,
                write_timeout=write_timeout,
                connect_timeout=connect_timeout,
                pool_timeout=pool_timeout,
                api_kwargs=api_kwargs,
                chat_id=None,
                message_id=None,
            )
        return await self._get_message().get_game_high_scores(
            user_id=user_id,
            read_timeout=read_timeout,
            write_timeout=write_timeout,
            connect_timeout=connect_timeout,
            pool_timeout=pool_timeout,
            api_kwargs=api_kwargs,
        )

    async def delete_message(
        self,
        *,
        read_timeout: ODVInput[float] = DEFAULT_NONE,
        write_timeout: ODVInput[float] = DEFAULT_NONE,
        connect_timeout: ODVInput[float] = DEFAULT_NONE,
        pool_timeout: ODVInput[float] = DEFAULT_NONE,
        api_kwargs: JSONDict | None = None,
    ) -> bool:
        """Shortcut for::

            await update.callback_query.message.delete(*args, **kwargs)

        For the documentation of the arguments, please see :meth:`telegram.Message.delete`.

        .. versionchanged:: 20.8
           Raises :exc:`TypeError` if :attr:`message` is not accessible.

        Returns:
            :obj:`bool`: On success, :obj:`True` is returned.

        Raises:
            :exc:`TypeError` if :attr:`message` is not accessible.

        """
        return await self._get_message(action="delete").delete(
            read_timeout=read_timeout,
            write_timeout=write_timeout,
            connect_timeout=connect_timeout,
            pool_timeout=pool_timeout,
            api_kwargs=api_kwargs,
        )

    async def pin_message(
        self,
        disable_notification: ODVInput[bool] = DEFAULT_NONE,
        *,
        read_timeout: ODVInput[float] = DEFAULT_NONE,
        write_timeout: ODVInput[float] = DEFAULT_NONE,
        connect_timeout: ODVInput[float] = DEFAULT_NONE,
        pool_timeout: ODVInput[float] = DEFAULT_NONE,
        api_kwargs: JSONDict | None = None,
    ) -> bool:
        """Shortcut for::

            await update.callback_query.message.pin(*args, **kwargs)

        For the documentation of the arguments, please see :meth:`telegram.Message.pin`.

        .. versionchanged:: 20.8
           Raises :exc:`TypeError` if :attr:`message` is not accessible.

        Returns:
            :obj:`bool`: On success, :obj:`True` is returned.

        Raises:
            :exc:`TypeError` if :attr:`message` is not accessible.
        """
        return await self._get_message(action="pin").pin(
            disable_notification=disable_notification,
            read_timeout=read_timeout,
            write_timeout=write_timeout,
            connect_timeout=connect_timeout,
            pool_timeout=pool_timeout,
            api_kwargs=api_kwargs,
        )

    async def unpin_message(
        self,
        *,
        read_timeout: ODVInput[float] = DEFAULT_NONE,
        write_timeout: ODVInput[float] = DEFAULT_NONE,
        connect_timeout: ODVInput[float] = DEFAULT_NONE,
        pool_timeout: ODVInput[float] = DEFAULT_NONE,
        api_kwargs: JSONDict | None = None,
    ) -> bool:
        """Shortcut for::

            await update.callback_query.message.unpin(*args, **kwargs)

        For the documentation of the arguments, please see :meth:`telegram.Message.unpin`.

        .. versionchanged:: 20.8
           Raises :exc:`TypeError` if :attr:`message` is not accessible.

        Returns:
            :obj:`bool`: On success, :obj:`True` is returned.

        Raises:
            :exc:`TypeError` if :attr:`message` is not accessible.
        """
        return await self._get_message(action="unpin").unpin(
            read_timeout=read_timeout,
            write_timeout=write_timeout,
            connect_timeout=connect_timeout,
            pool_timeout=pool_timeout,
            api_kwargs=api_kwargs,
        )

    async def copy_message(
        self,
        chat_id: int | str,
        caption: str | None = None,
        parse_mode: ODVInput[str] = DEFAULT_NONE,
        caption_entities: Sequence["MessageEntity"] | None = None,
        disable_notification: ODVInput[bool] = DEFAULT_NONE,
        reply_markup: "ReplyMarkup | None" = None,
        protect_content: ODVInput[bool] = DEFAULT_NONE,
        message_thread_id: int | None = None,
        reply_parameters: "ReplyParameters | None" = None,
        show_caption_above_media: bool | None = None,
        allow_paid_broadcast: bool | None = None,
        video_start_timestamp: int | None = None,
        suggested_post_parameters: "SuggestedPostParameters | None" = None,
        message_effect_id: str | None = None,
        *,
        allow_sending_without_reply: ODVInput[bool] = DEFAULT_NONE,
        reply_to_message_id: int | None = None,
        read_timeout: ODVInput[float] = DEFAULT_NONE,
        write_timeout: ODVInput[float] = DEFAULT_NONE,
        connect_timeout: ODVInput[float] = DEFAULT_NONE,
        pool_timeout: ODVInput[float] = DEFAULT_NONE,
        api_kwargs: JSONDict | None = None,
    ) -> "MessageId":
        """Shortcut for::

            await update.callback_query.message.copy(
                from_chat_id=update.message.chat_id,
                message_id=update.message.message_id,
                direct_messages_topic_id=update.message.direct_messages_topic.topic_id,
                *args,
                **kwargs
            )

        For the documentation of the arguments, please see :meth:`telegram.Message.copy`.

        .. versionchanged:: 20.8
           Raises :exc:`TypeError` if :attr:`message` is not accessible.

        Returns:
            :class:`telegram.MessageId`: On success, returns the MessageId of the sent message.

        Raises:
            :exc:`TypeError` if :attr:`message` is not accessible.
        """
        return await self._get_message(action="copy").copy(
            chat_id=chat_id,
            caption=caption,
            parse_mode=parse_mode,
            video_start_timestamp=video_start_timestamp,
            caption_entities=caption_entities,
            disable_notification=disable_notification,
            reply_to_message_id=reply_to_message_id,
            allow_sending_without_reply=allow_sending_without_reply,
            reply_markup=reply_markup,
            read_timeout=read_timeout,
            write_timeout=write_timeout,
            connect_timeout=connect_timeout,
            pool_timeout=pool_timeout,
            api_kwargs=api_kwargs,
            protect_content=protect_content,
            message_thread_id=message_thread_id,
            reply_parameters=reply_parameters,
            show_caption_above_media=show_caption_above_media,
            allow_paid_broadcast=allow_paid_broadcast,
            suggested_post_parameters=suggested_post_parameters,
            message_effect_id=message_effect_id,
        )

    MAX_ANSWER_TEXT_LENGTH: Final[int] = (
        constants.CallbackQueryLimit.ANSWER_CALLBACK_QUERY_TEXT_LENGTH
    )
    """
    :const:`telegram.constants.CallbackQueryLimit.ANSWER_CALLBACK_QUERY_TEXT_LENGTH`

    .. versionadded:: 13.2
    """



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains two objects that represent a Telegram bots (short) description."""

from telegram._telegramobject import TelegramObject
from telegram._utils.types import JSONDict


class BotDescription(TelegramObject):
    """This object represents the bot's description.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`description` is equal.

    .. versionadded:: 20.2

    Args:
        description (:obj:`str`): The bot's description.

    Attributes:
        description (:obj:`str`): The bot's description.

    """

    __slots__ = ("description",)

    def __init__(self, description: str, *, api_kwargs: JSONDict | None = None):
        super().__init__(api_kwargs=api_kwargs)
        self.description: str = description

        self._id_attrs = (self.description,)

        self._freeze()


class BotShortDescription(TelegramObject):
    """This object represents the bot's short description.

    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`short_description` is equal.

    .. versionadded:: 20.2

    Args:
        short_description (:obj:`str`): The bot's short description.

    Attributes:
        short_description (:obj:`str`): The bot's short description.

    """

    __slots__ = ("short_description",)

    def __init__(self, short_description: str, *, api_kwargs: JSONDict | None = None):
        super().__init__(api_kwargs=api_kwargs)
        self.short_description: str = short_description

        self._id_attrs = (self.short_description,)

        self._freeze()



#!/usr/bin/env python
#
#  A library that provides a Python interface to the Telegram Bot API
#  Copyright (C) 2015-2026
#  Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser Public License for more details.
#
#  You should have received a copy of the GNU Lesser Public License
#  along with this program.  If not, see [http://www.gnu.org/licenses/].
"""This module contains the CallbackDataCache class."""

import datetime as dtm
import time
from collections.abc import MutableMapping
from copy import copy
from typing import TYPE_CHECKING, Any, cast
from uuid import uuid4

try:
    from cachetools import LRUCache

    CACHE_TOOLS_AVAILABLE = True
except ImportError:
    CACHE_TOOLS_AVAILABLE = False


import contextlib

from telegram import CallbackQuery, InlineKeyboardMarkup, Message, User
from telegram._utils.datetime import to_float_timestamp
from telegram.error import TelegramError
from telegram.ext._utils.types import CDCData

if TYPE_CHECKING:
    from telegram.ext import ExtBot


class InvalidCallbackData(TelegramError):
    """
    Raised when the received callback data has been tampered with or deleted from cache.

    Examples:
        :any:`Arbitrary Callback Data Bot <examples.arbitrarycallbackdatabot>`

    .. seealso:: :wiki:`Arbitrary callback_data <Arbitrary-callback_data>`

    .. versionadded:: 13.6

    Args:
        callback_data (:obj:`int`, optional): The button data of which the callback data could not
            be found.

    Attributes:
        callback_data (:obj:`int`): Optional. The button data of which the callback data could not
            be found.
    """

    __slots__ = ("callback_data",)

    def __init__(self, callback_data: str | None = None) -> None:
        super().__init__(
            "The object belonging to this callback_data was deleted or the callback_data was "
            "manipulated."
        )
        self.callback_data: str | None = callback_data

    def __reduce__(self) -> tuple[type, tuple[str | None]]:  # type: ignore[override]
        """Defines how to serialize the exception for pickle. See
        :py:meth:`object.__reduce__` for more info.

        Returns:
            :obj:`tuple`
        """
        return self.__class__, (self.callback_data,)


class _KeyboardData:
    __slots__ = ("access_time", "button_data", "keyboard_uuid")

    def __init__(
        self,
        keyboard_uuid: str,
        access_time: float | None = None,
        button_data: dict[str, object] | None = None,
    ):
        self.keyboard_uuid = keyboard_uuid
        self.button_data = button_data or {}
        self.access_time = access_time or time.time()

    def update_access_time(self) -> None:
        """Updates the access time with the current time."""
        self.access_time = time.time()

    def to_tuple(self) -> tuple[str, float, dict[str, object]]:
        """Gives a tuple representation consisting of the keyboard uuid, the access time and the
        button data.
        """
        return self.keyboard_uuid, self.access_time, self.button_data


class CallbackDataCache:
    """A custom cache for storing the callback data of a :class:`telegram.ext.ExtBot`. Internally,
    it keeps two mappings with fixed maximum size:

    * One for mapping the data received in callback queries to the cached objects
    * One for mapping the IDs of received callback queries to the cached objects

    The second mapping allows to manually drop data that has been cached for keyboards of messages
    sent via inline mode.
    If necessary, will drop the least recently used items.

    Important:
        If you want to use this class, you must install PTB with the optional requirement
        ``callback-data``, i.e.

        .. code-block:: bash

           pip install "python-telegram-bot[callback-data]"

    Examples:
        :any:`Arbitrary Callback Data Bot <examples.arbitrarycallbackdatabot>`

    .. seealso:: :wiki:`Architecture Overview <Architecture>`,
        :wiki:`Arbitrary callback_data <Arbitrary-callback_data>`

    .. versionadded:: 13.6

    .. versionchanged:: 20.0
        To use this class, PTB must be installed via
        ``pip install "python-telegram-bot[callback-data]"``.

    Args:
        bot (:class:`telegram.ext.ExtBot`): The bot this cache is for.
        maxsize (:obj:`int`, optional): Maximum number of items in each of the internal mappings.
            Defaults to ``1024``.

        persistent_data (tuple[list[tuple[:obj:`str`, :obj:`float`, \
        dict[:obj:`str`, :class:`object`]]], dict[:obj:`str`, :obj:`str`]], optional): \
        Data to initialize the cache with, as returned by \
        :meth:`telegram.ext.BasePersistence.get_callback_data`.

    Attributes:
        bot (:class:`telegram.ext.ExtBot`): The bot this cache is for.

    """

    __slots__ = ("_callback_queries", "_keyboard_data", "_maxsize", "bot")

    def __init__(
        self,
        bot: "ExtBot[Any]",
        maxsize: int = 1024,
        persistent_data: CDCData | None = None,
    ):
        if not CACHE_TOOLS_AVAILABLE:
            raise RuntimeError(
                "To use `CallbackDataCache`, PTB must be installed via `pip install "
                '"python-telegram-bot[callback-data]"`.'
            )

        self.bot: ExtBot[Any] = bot
        self._maxsize: int = maxsize
        self._keyboard_data: MutableMapping[str, _KeyboardData] = LRUCache(maxsize=maxsize)
        self._callback_queries: MutableMapping[str, str] = LRUCache(maxsize=maxsize)

        if persistent_data:
            self.load_persistence_data(persistent_data)

    def load_persistence_data(self, persistent_data: CDCData) -> None:
        """Loads data into the cache.

        Warning:
            This method is not intended to be called by users directly.

        .. versionadded:: 20.0

        Args:
            persistent_data (tuple[list[tuple[:obj:`str`, :obj:`float`, \
            dict[:obj:`str`, :class:`object`]]], dict[:obj:`str`, :obj:`str`]], optional): \
            Data to load, as returned by \
            :meth:`telegram.ext.BasePersistence.get_callback_data`.
        """
        keyboard_data, callback_queries = persistent_data
        for key, value in callback_queries.items():
            self._callback_queries[key] = value
        for uuid, access_time, data in keyboard_data:
            self._keyboard_data[uuid] = _KeyboardData(
                keyboard_uuid=uuid, access_time=access_time, button_data=data
            )

    @property
    def maxsize(self) -> int:
        """:obj:`int`: The maximum size of the cache.

        .. versionchanged:: 20.0
           This property is now read-only.
        """
        return self._maxsize

    @property
    def persistence_data(self) -> CDCData:
        """tuple[list[tuple[:obj:`str`, :obj:`float`, dict[:obj:`str`, :class:`object`]]],
        dict[:obj:`str`, :obj:`str`]]: The data that needs to be persisted to allow
        caching callback data across bot reboots.
        """
        # While building a list/dict from the LRUCaches has linear runtime (in the number of
        # entries), the runtime is bounded by maxsize and it has the big upside of not throwing a
        # highly customized data structure at users trying to implement a custom persistence class
        return [data.to_tuple() for data in self._keyboard_data.values()], dict(
            self._callback_queries.items()
        )

    def process_keyboard(self, reply_markup: InlineKeyboardMarkup) -> InlineKeyboardMarkup:
        """Registers the reply markup to the cache. If any of the buttons have
        :attr:`~telegram.InlineKeyboardButton.callback_data`, stores that data and builds a new
        keyboard with the correspondingly replaced buttons. Otherwise, does nothing and returns
        the original reply markup.

        Args:
            reply_markup (:class:`telegram.InlineKeyboardMarkup`): The keyboard.

        Returns:
            :class:`telegram.InlineKeyboardMarkup`: The keyboard to be passed to Telegram.

        """
        keyboard_uuid = uuid4().hex
        keyboard_data = _KeyboardData(keyboard_uuid)

        # Built a new nested list of buttons by replacing the callback data if needed
        buttons = []

        for column in reply_markup.inline_keyboard:
            cols = []
            for btn in column:
                if btn.callback_data is not None:
                    # We create a new button instead of replacing callback_data in case the
                    # same object is used elsewhere
                    btn_copy = copy(btn)
                    btn_copy.update_callback_data(
                        self.__put_button(btn.callback_data, keyboard_data)
                    )
                    cols.append(btn_copy)
                else:
                    cols.append(btn)
            buttons.append(cols)

        if not keyboard_data.button_data:
            # If we arrive here, no data had to be replaced and we can return the input
            return reply_markup

        self._keyboard_data[keyboard_uuid] = keyboard_data
        return InlineKeyboardMarkup(buttons)

    @staticmethod
    def __put_button(callback_data: object, keyboard_data: _KeyboardData) -> str:
        """Stores the data for a single button in :attr:`keyboard_data`.
        Returns the string that should be passed instead of the callback_data, which is
        ``keyboard_uuid + button_uuids``.
        """
        uuid = uuid4().hex
        keyboard_data.button_data[uuid] = callback_data
        return f"{keyboard_data.keyboard_uuid}{uuid}"

    def __get_keyboard_uuid_and_button_data(
        self, callback_data: str
    ) -> tuple[str, object] | tuple[None, InvalidCallbackData]:
        keyboard, button = self.extract_uuids(callback_data)
        try:
            # we get the values before calling update() in case KeyErrors are raised
            # we don't want to update in that case
            keyboard_data = self._keyboard_data[keyboard]
            button_data = keyboard_data.button_data[button]
            # Update the timestamp for the LRU
            keyboard_data.update_access_time()
        except KeyError:
            return None, InvalidCallbackData(callback_data)
        return keyboard, button_data

    @staticmethod
    def extract_uuids(callback_data: str) -> tuple[str, str]:
        """Extracts the keyboard uuid and the button uuid from the given :paramref:`callback_data`.

        Args:
            callback_data (:obj:`str`): The
                :paramref:`~telegram.InlineKeyboardButton.callback_data` as present in the button.

        Returns:
            (:obj:`str`, :obj:`str`): Tuple of keyboard and button uuid

        """
        # Extract the uuids as put in __put_button
        return callback_data[:32], callback_data[32:]

    def process_message(self, message: Message) -> None:
        """Replaces the data in the inline keyboard attached to the message with the cached
        objects, if necessary. If the data could not be found,
        :class:`telegram.ext.InvalidCallbackData` will be inserted.

        Note:
            Checks :attr:`telegram.Message.via_bot` and :attr:`telegram.Message.from_user` to check
            if the reply markup (if any) was actually sent by this cache's bot. If it was not, the
            message will be returned unchanged.

            Note that this will fail for channel posts, as :attr:`telegram.Message.from_user` is
            :obj:`None` for those! In the corresponding reply markups the callback data will be
            replaced by :class:`telegram.ext.InvalidCallbackData`.

        Warning:
            * Does *not* consider :attr:`telegram.Message.reply_to_message` and
              :attr:`telegram.Message.pinned_message`. Pass them to this method separately.
            * *In place*, i.e. the passed :class:`telegram.Message` will be changed!

        Args:
            message (:class:`telegram.Message`): The message.

        """
        self.__process_message(message)

    def __process_message(self, message: Message) -> str | None:
        """As documented in process_message, but returns the uuid of the attached keyboard, if any,
        which is relevant for process_callback_query.

        **IN PLACE**
        """
        if not message.reply_markup:
            return None

        if message.via_bot:
            sender: User | None = message.via_bot
        elif message.from_user:
            sender = message.from_user
        else:
            sender = None

        if sender is not None and sender != self.bot.bot:
            return None

        keyboard_uuid = None

        for row in message.reply_markup.inline_keyboard:
            for button in row:
                if button.callback_data:
                    button_data = cast("str", button.callback_data)
                    keyboard_id, callback_data = self.__get_keyboard_uuid_and_button_data(
                        button_data
                    )
                    # update_callback_data makes sure that the _id_attrs are updated
                    button.update_callback_data(callback_data)

                    # This is lazy loaded. The firsts time we find a button
                    # we load the associated keyboard - afterwards, there is
                    if not keyboard_uuid and not isinstance(callback_data, InvalidCallbackData):
                        keyboard_uuid = keyboard_id

        return keyboard_uuid

    def process_callback_query(self, callback_query: CallbackQuery) -> None:
        """Replaces the data in the callback query and the attached messages keyboard with the
        cached objects, if necessary. If the data could not be found,
        :class:`telegram.ext.InvalidCallbackData` will be inserted.
        If :attr:`telegram.CallbackQuery.data` or :attr:`telegram.CallbackQuery.message` is
        present, this also saves the callback queries ID in order to be able to resolve it to the
        stored data.

        Note:
            Also considers inserts data into the buttons of
            :attr:`telegram.Message.reply_to_message` and :attr:`telegram.Message.pinned_message`
            if necessary.

        Warning:
            *In place*, i.e. the passed :class:`telegram.CallbackQuery` will be changed!

        Args:
            callback_query (:class:`telegram.CallbackQuery`): The callback query.

        """
        mapped = False

        if callback_query.data:
            data = callback_query.data

            # Get the cached callback data for the CallbackQuery
            keyboard_uuid, button_data = self.__get_keyboard_uuid_and_button_data(data)
            with callback_query._unfrozen():
                callback_query.data = button_data  # type: ignore[assignment]

            # Map the callback queries ID to the keyboards UUID for later use
            if not mapped and not isinstance(button_data, InvalidCallbackData):
                self._callback_queries[callback_query.id] = keyboard_uuid  # type: ignore
                mapped = True

        # Get the cached callback data for the inline keyboard attached to the
        # CallbackQuery.
        if isinstance(callback_query.message, Message):
            self.__process_message(callback_query.message)
            for maybe_message in (
                callback_query.message.pinned_message,
                callback_query.message.reply_to_message,
            ):
                if isinstance(maybe_message, Message):
                    self.__process_message(maybe_message)

    def drop_data(self, callback_query: CallbackQuery) -> None:
        """Deletes the data for the specified callback query.

        Note:
            Will *not* raise exceptions in case the callback data is not found in the cache.
            *Will* raise :exc:`KeyError` in case the callback query can not be found in the
            cache.

        Args:
            callback_query (:class:`telegram.CallbackQuery`): The callback query.

        Raises:
            KeyError: If the callback query can not be found in the cache
        """
        try:
            keyboard_uuid = self._callback_queries.pop(callback_query.id)
            self.__drop_keyboard(keyboard_uuid)
        except KeyError as exc:
            raise KeyError("CallbackQuery was not found in cache.") from exc

    def __drop_keyboard(self, keyboard_uuid: str) -> None:
        with contextlib.suppress(KeyError):
            self._keyboard_data.pop(keyboard_uuid)

    def clear_callback_data(self, time_cutoff: float | dtm.datetime | None = None) -> None:
        """Clears the stored callback data.

        Args:
            time_cutoff (:obj:`float` | :obj:`datetime.datetime`, optional): Pass a UNIX timestamp
                or a :obj:`datetime.datetime` to clear only entries which are older.
                |tz-naive-dtms|

        """
        self.__clear(self._keyboard_data, time_cutoff=time_cutoff)

    def clear_callback_queries(self) -> None:
        """Clears the stored callback query IDs."""
        self.__clear(self._callback_queries)

    def __clear(
        self, mapping: MutableMapping, time_cutoff: float | dtm.datetime | None = None
    ) -> None:
        if not time_cutoff:
            mapping.clear()
            return

        if isinstance(time_cutoff, dtm.datetime):
            effective_cutoff = to_float_timestamp(
                time_cutoff, tzinfo=self.bot.defaults.tzinfo if self.bot.defaults else None
            )
        else:
            effective_cutoff = time_cutoff

        # We need a list instead of a generator here, as the list doesn't change it's size
        # during the iteration
        to_drop = [key for key, data in mapping.items() if data.access_time < effective_cutoff]
        for key in to_drop:
            mapping.pop(key)



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
import pytest

from telegram import Bot, InlineKeyboardButton, Update, User
from telegram.helpers import escape_markdown
from tests.auxil.bot_method_checks import (
    check_defaults_handling,
    check_shortcut_call,
    check_shortcut_signature,
)
from tests.auxil.slots import mro_slots


@pytest.fixture(scope="module")
def json_dict():
    return {
        "id": UserTestBase.id_,
        "is_bot": UserTestBase.is_bot,
        "first_name": UserTestBase.first_name,
        "last_name": UserTestBase.last_name,
        "username": UserTestBase.username,
        "language_code": UserTestBase.language_code,
        "can_join_groups": UserTestBase.can_join_groups,
        "can_read_all_group_messages": UserTestBase.can_read_all_group_messages,
        "supports_inline_queries": UserTestBase.supports_inline_queries,
        "is_premium": UserTestBase.is_premium,
        "added_to_attachment_menu": UserTestBase.added_to_attachment_menu,
        "can_connect_to_business": UserTestBase.can_connect_to_business,
        "has_main_web_app": UserTestBase.has_main_web_app,
        "has_topics_enabled": UserTestBase.has_topics_enabled,
        "allows_users_to_create_topics": UserTestBase.allows_users_to_create_topics,
    }


@pytest.fixture
def user(bot):
    user = User(
        id=UserTestBase.id_,
        first_name=UserTestBase.first_name,
        is_bot=UserTestBase.is_bot,
        last_name=UserTestBase.last_name,
        username=UserTestBase.username,
        language_code=UserTestBase.language_code,
        can_join_groups=UserTestBase.can_join_groups,
        can_read_all_group_messages=UserTestBase.can_read_all_group_messages,
        supports_inline_queries=UserTestBase.supports_inline_queries,
        is_premium=UserTestBase.is_premium,
        added_to_attachment_menu=UserTestBase.added_to_attachment_menu,
        can_connect_to_business=UserTestBase.can_connect_to_business,
        has_main_web_app=UserTestBase.has_main_web_app,
        has_topics_enabled=UserTestBase.has_topics_enabled,
        allows_users_to_create_topics=UserTestBase.allows_users_to_create_topics,
    )
    user.set_bot(bot)
    user._unfreeze()
    return user


class UserTestBase:
    id_ = 1
    is_bot = True
    first_name = "first\u2022name"
    last_name = "last\u2022name"
    username = "username"
    language_code = "en_us"
    can_join_groups = True
    can_read_all_group_messages = True
    supports_inline_queries = False
    is_premium = True
    added_to_attachment_menu = False
    can_connect_to_business = True
    has_main_web_app = False
    has_topics_enabled = False
    allows_users_to_create_topics = False


class TestUserWithoutRequest(UserTestBase):
    def test_slot_behaviour(self, user):
        for attr in user.__slots__:
            assert getattr(user, attr, "err") != "err", f"got extra slot '{attr}'"
        assert len(mro_slots(user)) == len(set(mro_slots(user))), "duplicate slot"

    def test_de_json(self, json_dict, offline_bot):
        user = User.de_json(json_dict, offline_bot)
        assert user.api_kwargs == {}

        assert user.id == self.id_
        assert user.is_bot == self.is_bot
        assert user.first_name == self.first_name
        assert user.last_name == self.last_name
        assert user.username == self.username
        assert user.language_code == self.language_code
        assert user.can_join_groups == self.can_join_groups
        assert user.can_read_all_group_messages == self.can_read_all_group_messages
        assert user.supports_inline_queries == self.supports_inline_queries
        assert user.is_premium == self.is_premium
        assert user.added_to_attachment_menu == self.added_to_attachment_menu
        assert user.can_connect_to_business == self.can_connect_to_business
        assert user.has_main_web_app == self.has_main_web_app
        assert user.has_topics_enabled == self.has_topics_enabled
        assert user.allows_users_to_create_topics == self.allows_users_to_create_topics

    def test_to_dict(self, user):
        user_dict = user.to_dict()

        assert isinstance(user_dict, dict)
        assert user_dict["id"] == user.id
        assert user_dict["is_bot"] == user.is_bot
        assert user_dict["first_name"] == user.first_name
        assert user_dict["last_name"] == user.last_name
        assert user_dict["username"] == user.username
        assert user_dict["language_code"] == user.language_code
        assert user_dict["can_join_groups"] == user.can_join_groups
        assert user_dict["can_read_all_group_messages"] == user.can_read_all_group_messages
        assert user_dict["supports_inline_queries"] == user.supports_inline_queries
        assert user_dict["is_premium"] == user.is_premium
        assert user_dict["added_to_attachment_menu"] == user.added_to_attachment_menu
        assert user_dict["can_connect_to_business"] == user.can_connect_to_business
        assert user_dict["has_main_web_app"] == user.has_main_web_app
        assert user_dict["has_topics_enabled"] == user.has_topics_enabled
        assert user_dict["allows_users_to_create_topics"] == user.allows_users_to_create_topics

    def test_equality(self):
        a = User(self.id_, self.first_name, self.is_bot, self.last_name)
        b = User(self.id_, self.first_name, self.is_bot, self.last_name)
        c = User(self.id_, self.first_name, self.is_bot)
        d = User(0, self.first_name, self.is_bot, self.last_name)
        e = Update(self.id_)

        assert a == b
        assert hash(a) == hash(b)
        assert a is not b

        assert a == c
        assert hash(a) == hash(c)

        assert a != d
        assert hash(a) != hash(d)

        assert a != e
        assert hash(a) != hash(e)

    def test_name(self, user):
        assert user.name == "@username"
        user.username = None
        assert user.name == "first\u2022name last\u2022name"
        user.last_name = None
        assert user.name == "first\u2022name"
        user.username = self.username
        assert user.name == "@username"

    def test_full_name(self, user):
        assert user.full_name == "first\u2022name last\u2022name"
        user.last_name = None
        assert user.full_name == "first\u2022name"

    def test_link(self, user):
        assert user.link == f"https://t.me/{user.username}"
        user.username = None
        assert user.link is None

    async def test_instance_method_get_profile_photos(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["user_id"] == user.id

        assert check_shortcut_signature(
            User.get_profile_photos, Bot.get_user_profile_photos, ["user_id"], []
        )
        assert await check_shortcut_call(
            user.get_profile_photos, user.get_bot(), "get_user_profile_photos"
        )
        assert await check_defaults_handling(user.get_profile_photos, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "get_user_profile_photos", make_assertion)
        assert await user.get_profile_photos()

    async def test_instance_method_get_profile_audios(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["user_id"] == user.id

        assert check_shortcut_signature(
            User.get_profile_audios, Bot.get_user_profile_audios, ["user_id"], []
        )
        assert await check_shortcut_call(
            user.get_profile_audios, user.get_bot(), "get_user_profile_audios"
        )
        assert await check_defaults_handling(user.get_profile_audios, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "get_user_profile_audios", make_assertion)
        assert await user.get_profile_audios()

    async def test_instance_method_pin_message(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id

        assert check_shortcut_signature(User.pin_message, Bot.pin_chat_message, ["chat_id"], [])
        assert await check_shortcut_call(user.pin_message, user.get_bot(), "pin_chat_message")
        assert await check_defaults_handling(user.pin_message, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "pin_chat_message", make_assertion)
        assert await user.pin_message(1)

    async def test_instance_method_unpin_message(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id

        assert check_shortcut_signature(
            User.unpin_message, Bot.unpin_chat_message, ["chat_id"], []
        )
        assert await check_shortcut_call(user.unpin_message, user.get_bot(), "unpin_chat_message")
        assert await check_defaults_handling(user.unpin_message, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "unpin_chat_message", make_assertion)
        assert await user.unpin_message()

    async def test_instance_method_unpin_all_messages(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id

        assert check_shortcut_signature(
            User.unpin_all_messages, Bot.unpin_all_chat_messages, ["chat_id"], []
        )
        assert await check_shortcut_call(
            user.unpin_all_messages, user.get_bot(), "unpin_all_chat_messages"
        )
        assert await check_defaults_handling(user.unpin_all_messages, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "unpin_all_chat_messages", make_assertion)
        assert await user.unpin_all_messages()

    async def test_instance_method_send_message(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id and kwargs["text"] == "test"

        assert check_shortcut_signature(User.send_message, Bot.send_message, ["chat_id"], [])
        assert await check_shortcut_call(user.send_message, user.get_bot(), "send_message")
        assert await check_defaults_handling(user.send_message, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "send_message", make_assertion)
        assert await user.send_message("test")

    async def test_instance_method_send_message_draft(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return (
                kwargs["chat_id"] == user.id
                and kwargs["draft_id"] == 123
                and kwargs["text"] == "test"
            )

        assert check_shortcut_signature(
            User.send_message_draft, Bot.send_message_draft, ["chat_id"], []
        )
        assert await check_shortcut_call(
            user.send_message_draft, user.get_bot(), "send_message_draft"
        )
        assert await check_defaults_handling(user.send_message_draft, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "send_message_draft", make_assertion)
        assert await user.send_message_draft(123, "test")

    async def test_instance_method_send_photo(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id and kwargs["photo"] == "test_photo"

        assert check_shortcut_signature(User.send_photo, Bot.send_photo, ["chat_id"], [])
        assert await check_shortcut_call(user.send_photo, user.get_bot(), "send_photo")
        assert await check_defaults_handling(user.send_photo, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "send_photo", make_assertion)
        assert await user.send_photo("test_photo")

    async def test_instance_method_send_media_group(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id and kwargs["media"] == "test_media_group"

        assert check_shortcut_signature(
            User.send_media_group, Bot.send_media_group, ["chat_id"], []
        )
        assert await check_shortcut_call(user.send_media_group, user.get_bot(), "send_media_group")
        assert await check_defaults_handling(user.send_media_group, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "send_media_group", make_assertion)
        assert await user.send_media_group("test_media_group")

    async def test_instance_method_send_audio(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id and kwargs["audio"] == "test_audio"

        assert check_shortcut_signature(User.send_audio, Bot.send_audio, ["chat_id"], [])
        assert await check_shortcut_call(user.send_audio, user.get_bot(), "send_audio")
        assert await check_defaults_handling(user.send_audio, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "send_audio", make_assertion)
        assert await user.send_audio("test_audio")

    async def test_instance_method_send_chat_action(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id and kwargs["action"] == "test_chat_action"

        assert check_shortcut_signature(
            User.send_chat_action, Bot.send_chat_action, ["chat_id"], []
        )
        assert await check_shortcut_call(user.send_chat_action, user.get_bot(), "send_chat_action")
        assert await check_defaults_handling(user.send_chat_action, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "send_chat_action", make_assertion)
        assert await user.send_chat_action("test_chat_action")

    async def test_instance_method_send_contact(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id and kwargs["phone_number"] == "test_contact"

        assert check_shortcut_signature(User.send_contact, Bot.send_contact, ["chat_id"], [])
        assert await check_shortcut_call(user.send_contact, user.get_bot(), "send_contact")
        assert await check_defaults_handling(user.send_contact, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "send_contact", make_assertion)
        assert await user.send_contact(phone_number="test_contact")

    async def test_instance_method_send_dice(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id and kwargs["emoji"] == "test_dice"

        assert check_shortcut_signature(User.send_dice, Bot.send_dice, ["chat_id"], [])
        assert await check_shortcut_call(user.send_dice, user.get_bot(), "send_dice")
        assert await check_defaults_handling(user.send_dice, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "send_dice", make_assertion)
        assert await user.send_dice(emoji="test_dice")

    async def test_instance_method_send_document(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id and kwargs["document"] == "test_document"

        assert check_shortcut_signature(User.send_document, Bot.send_document, ["chat_id"], [])
        assert await check_shortcut_call(user.send_document, user.get_bot(), "send_document")
        assert await check_defaults_handling(user.send_document, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "send_document", make_assertion)
        assert await user.send_document("test_document")

    async def test_instance_method_send_game(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id and kwargs["game_short_name"] == "test_game"

        assert check_shortcut_signature(User.send_game, Bot.send_game, ["chat_id"], [])
        assert await check_shortcut_call(user.send_game, user.get_bot(), "send_game")
        assert await check_defaults_handling(user.send_game, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "send_game", make_assertion)
        assert await user.send_game(game_short_name="test_game")

    async def test_instance_method_send_invoice(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            title = kwargs["title"] == "title"
            description = kwargs["description"] == "description"
            payload = kwargs["payload"] == "payload"
            provider_token = kwargs["provider_token"] == "provider_token"
            currency = kwargs["currency"] == "currency"
            prices = kwargs["prices"] == "prices"
            args = title and description and payload and provider_token and currency and prices
            return kwargs["chat_id"] == user.id and args

        assert check_shortcut_signature(User.send_invoice, Bot.send_invoice, ["chat_id"], [])
        assert await check_shortcut_call(user.send_invoice, user.get_bot(), "send_invoice")
        assert await check_defaults_handling(user.send_invoice, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "send_invoice", make_assertion)
        assert await user.send_invoice(
            "title",
            "description",
            "payload",
            "currency",
            "prices",
            "provider_token",
        )

    async def test_instance_method_send_location(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id and kwargs["latitude"] == "test_location"

        assert check_shortcut_signature(User.send_location, Bot.send_location, ["chat_id"], [])
        assert await check_shortcut_call(user.send_location, user.get_bot(), "send_location")
        assert await check_defaults_handling(user.send_location, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "send_location", make_assertion)
        assert await user.send_location("test_location")

    async def test_instance_method_send_sticker(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id and kwargs["sticker"] == "test_sticker"

        assert check_shortcut_signature(User.send_sticker, Bot.send_sticker, ["chat_id"], [])
        assert await check_shortcut_call(user.send_sticker, user.get_bot(), "send_sticker")
        assert await check_defaults_handling(user.send_sticker, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "send_sticker", make_assertion)
        assert await user.send_sticker("test_sticker")

    async def test_instance_method_send_video(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id and kwargs["video"] == "test_video"

        assert check_shortcut_signature(User.send_video, Bot.send_video, ["chat_id"], [])
        assert await check_shortcut_call(user.send_video, user.get_bot(), "send_video")
        assert await check_defaults_handling(user.send_video, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "send_video", make_assertion)
        assert await user.send_video("test_video")

    async def test_instance_method_send_venue(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id and kwargs["title"] == "test_venue"

        assert check_shortcut_signature(User.send_venue, Bot.send_venue, ["chat_id"], [])
        assert await check_shortcut_call(user.send_venue, user.get_bot(), "send_venue")
        assert await check_defaults_handling(user.send_venue, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "send_venue", make_assertion)
        assert await user.send_venue(title="test_venue")

    async def test_instance_method_send_video_note(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id and kwargs["video_note"] == "test_video_note"

        assert check_shortcut_signature(User.send_video_note, Bot.send_video_note, ["chat_id"], [])
        assert await check_shortcut_call(user.send_video_note, user.get_bot(), "send_video_note")
        assert await check_defaults_handling(user.send_video_note, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "send_video_note", make_assertion)
        assert await user.send_video_note("test_video_note")

    async def test_instance_method_send_voice(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id and kwargs["voice"] == "test_voice"

        assert check_shortcut_signature(User.send_voice, Bot.send_voice, ["chat_id"], [])
        assert await check_shortcut_call(user.send_voice, user.get_bot(), "send_voice")
        assert await check_defaults_handling(user.send_voice, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "send_voice", make_assertion)
        assert await user.send_voice("test_voice")

    async def test_instance_method_send_animation(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id and kwargs["animation"] == "test_animation"

        assert check_shortcut_signature(User.send_animation, Bot.send_animation, ["chat_id"], [])
        assert await check_shortcut_call(user.send_animation, user.get_bot(), "send_animation")
        assert await check_defaults_handling(user.send_animation, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "send_animation", make_assertion)
        assert await user.send_animation("test_animation")

    async def test_instance_method_send_poll(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id and kwargs["question"] == "test_poll"

        assert check_shortcut_signature(User.send_poll, Bot.send_poll, ["chat_id"], [])
        assert await check_shortcut_call(user.send_poll, user.get_bot(), "send_poll")
        assert await check_defaults_handling(user.send_poll, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "send_poll", make_assertion)
        assert await user.send_poll(question="test_poll", options=[1, 2])

    async def test_instance_method_send_copy(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            user_id = kwargs["chat_id"] == user.id
            message_id = kwargs["message_id"] == "message_id"
            from_chat_id = kwargs["from_chat_id"] == "from_chat_id"
            return from_chat_id and message_id and user_id

        assert check_shortcut_signature(User.send_copy, Bot.copy_message, ["chat_id"], [])
        assert await check_shortcut_call(user.send_copy, user.get_bot(), "copy_message")
        assert await check_defaults_handling(user.send_copy, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "copy_message", make_assertion)
        assert await user.send_copy(from_chat_id="from_chat_id", message_id="message_id")

    async def test_instance_method_copy_message(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            chat_id = kwargs["chat_id"] == "chat_id"
            message_id = kwargs["message_id"] == "message_id"
            user_id = kwargs["from_chat_id"] == user.id
            return chat_id and message_id and user_id

        assert check_shortcut_signature(User.copy_message, Bot.copy_message, ["from_chat_id"], [])
        assert await check_shortcut_call(user.copy_message, user.get_bot(), "copy_message")
        assert await check_defaults_handling(user.copy_message, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "copy_message", make_assertion)
        assert await user.copy_message(chat_id="chat_id", message_id="message_id")

    async def test_instance_method_get_user_chat_boosts(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            chat_id = kwargs["chat_id"] == "chat_id"
            user_id = kwargs["user_id"] == user.id
            return chat_id and user_id

        assert check_shortcut_signature(
            User.get_chat_boosts, Bot.get_user_chat_boosts, ["user_id"], []
        )
        assert await check_shortcut_call(
            user.get_chat_boosts, user.get_bot(), "get_user_chat_boosts"
        )
        assert await check_defaults_handling(user.get_chat_boosts, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "get_user_chat_boosts", make_assertion)
        assert await user.get_chat_boosts(chat_id="chat_id")

    async def test_instance_method_get_menu_button(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id

        assert check_shortcut_signature(
            User.get_menu_button, Bot.get_chat_menu_button, ["chat_id"], []
        )
        assert await check_shortcut_call(
            user.get_menu_button,
            user.get_bot(),
            "get_chat_menu_button",
            shortcut_kwargs=["chat_id"],
        )
        assert await check_defaults_handling(user.get_menu_button, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "get_chat_menu_button", make_assertion)
        assert await user.get_menu_button()

    async def test_instance_method_set_menu_button(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id and kwargs["menu_button"] == "menu_button"

        assert check_shortcut_signature(
            User.set_menu_button, Bot.set_chat_menu_button, ["chat_id"], []
        )
        assert await check_shortcut_call(
            user.set_menu_button,
            user.get_bot(),
            "set_chat_menu_button",
            shortcut_kwargs=["chat_id"],
        )
        assert await check_defaults_handling(user.set_menu_button, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "set_chat_menu_button", make_assertion)
        assert await user.set_menu_button(menu_button="menu_button")

    async def test_instance_method_approve_join_request(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            chat_id = kwargs["chat_id"] == "chat_id"
            user_id = kwargs["user_id"] == user.id
            return chat_id and user_id

        assert check_shortcut_signature(
            User.approve_join_request, Bot.approve_chat_join_request, ["user_id"], []
        )
        assert await check_shortcut_call(
            user.approve_join_request, user.get_bot(), "approve_chat_join_request"
        )
        assert await check_defaults_handling(user.approve_join_request, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "approve_chat_join_request", make_assertion)
        assert await user.approve_join_request(chat_id="chat_id")

    async def test_instance_method_decline_join_request(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            chat_id = kwargs["chat_id"] == "chat_id"
            user_id = kwargs["user_id"] == user.id
            return chat_id and user_id

        assert check_shortcut_signature(
            User.decline_join_request, Bot.decline_chat_join_request, ["user_id"], []
        )
        assert await check_shortcut_call(
            user.decline_join_request, user.get_bot(), "decline_chat_join_request"
        )
        assert await check_defaults_handling(user.decline_join_request, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "decline_chat_join_request", make_assertion)
        assert await user.decline_join_request(chat_id="chat_id")

    async def test_mention_html(self, user):
        expected = '<a href="tg://user?id={}">{}</a>'

        assert user.mention_html() == expected.format(user.id, user.full_name)
        assert user.mention_html("the<b>name\u2022") == expected.format(
            user.id, "the&lt;b&gt;name\u2022"
        )
        assert user.mention_html(user.username) == expected.format(user.id, user.username)

    def test_mention_button(self, user):
        expected_name = InlineKeyboardButton(text="Bob", url=f"tg://user?id={user.id}")
        expected_full = InlineKeyboardButton(text=user.full_name, url=f"tg://user?id={user.id}")

        assert user.mention_button("Bob") == expected_name
        assert user.mention_button() == expected_full

    def test_mention_markdown(self, user):
        expected = "[{}](tg://user?id={})"

        assert user.mention_markdown() == expected.format(user.full_name, user.id)
        assert user.mention_markdown("the_name*\u2022") == expected.format(
            "the_name*\u2022", user.id
        )
        assert user.mention_markdown(user.username) == expected.format(user.username, user.id)

    async def test_mention_markdown_v2(self, user):
        user.first_name = "first{name"
        user.last_name = "last_name"

        expected = "[{}](tg://user?id={})"

        assert user.mention_markdown_v2() == expected.format(
            escape_markdown(user.full_name, version=2), user.id
        )
        assert user.mention_markdown_v2("the{name>\u2022") == expected.format(
            "the\\{name\\>\u2022", user.id
        )
        assert user.mention_markdown_v2(user.username) == expected.format(user.username, user.id)

    async def test_delete_message(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id and kwargs["message_id"] == 42

        assert check_shortcut_signature(user.delete_message, Bot.delete_message, ["chat_id"], [])
        assert await check_shortcut_call(user.delete_message, user.get_bot(), "delete_message")
        assert await check_defaults_handling(user.delete_message, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "delete_message", make_assertion)
        assert await user.delete_message(message_id=42)

    async def test_delete_messages(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["chat_id"] == user.id and kwargs["message_ids"] == (42, 43)

        assert check_shortcut_signature(user.delete_messages, Bot.delete_messages, ["chat_id"], [])
        assert await check_shortcut_call(user.delete_messages, user.get_bot(), "delete_messages")
        assert await check_defaults_handling(user.delete_messages, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "delete_messages", make_assertion)
        assert await user.delete_messages(message_ids=(42, 43))

    async def test_instance_method_send_copies(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            from_chat_id = kwargs["from_chat_id"] == "test_copies"
            message_ids = kwargs["message_ids"] == (42, 43)
            user_id = kwargs["chat_id"] == user.id
            return from_chat_id and message_ids and user_id

        assert check_shortcut_signature(user.send_copies, Bot.copy_messages, ["chat_id"], [])
        assert await check_shortcut_call(user.send_copies, user.get_bot(), "copy_messages")
        assert await check_defaults_handling(user.send_copies, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "copy_messages", make_assertion)
        assert await user.send_copies(from_chat_id="test_copies", message_ids=(42, 43))

    async def test_instance_method_copy_messages(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            from_chat_id = kwargs["from_chat_id"] == user.id
            message_ids = kwargs["message_ids"] == (42, 43)
            user_id = kwargs["chat_id"] == "test_copies"
            return from_chat_id and message_ids and user_id

        assert check_shortcut_signature(
            user.copy_messages, Bot.copy_messages, ["from_chat_id"], []
        )
        assert await check_shortcut_call(user.copy_messages, user.get_bot(), "copy_messages")
        assert await check_defaults_handling(user.copy_messages, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "copy_messages", make_assertion)
        assert await user.copy_messages(chat_id="test_copies", message_ids=(42, 43))

    async def test_instance_method_forward_from(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            user_id = kwargs["chat_id"] == user.id
            message_id = kwargs["message_id"] == 42
            from_chat_id = kwargs["from_chat_id"] == "test_forward"
            return from_chat_id and message_id and user_id

        assert check_shortcut_signature(user.forward_from, Bot.forward_message, ["chat_id"], [])
        assert await check_shortcut_call(user.forward_from, user.get_bot(), "forward_message")
        assert await check_defaults_handling(user.forward_from, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "forward_message", make_assertion)
        assert await user.forward_from(from_chat_id="test_forward", message_id=42)

    async def test_instance_method_forward_to(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            from_chat_id = kwargs["from_chat_id"] == user.id
            message_id = kwargs["message_id"] == 42
            user_id = kwargs["chat_id"] == "test_forward"
            return from_chat_id and message_id and user_id

        assert check_shortcut_signature(user.forward_to, Bot.forward_message, ["from_chat_id"], [])
        assert await check_shortcut_call(user.forward_to, user.get_bot(), "forward_message")
        assert await check_defaults_handling(user.forward_to, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "forward_message", make_assertion)
        assert await user.forward_to(chat_id="test_forward", message_id=42)

    async def test_instance_method_forward_messages_from(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            user_id = kwargs["chat_id"] == user.id
            message_ids = kwargs["message_ids"] == (42, 43)
            from_chat_id = kwargs["from_chat_id"] == "test_forwards"
            return from_chat_id and message_ids and user_id

        assert check_shortcut_signature(
            user.forward_messages_from, Bot.forward_messages, ["chat_id"], []
        )
        assert await check_shortcut_call(
            user.forward_messages_from, user.get_bot(), "forward_messages"
        )
        assert await check_defaults_handling(user.forward_messages_from, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "forward_messages", make_assertion)
        assert await user.forward_messages_from(from_chat_id="test_forwards", message_ids=(42, 43))

    async def test_instance_method_forward_messages_to(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            from_chat_id = kwargs["from_chat_id"] == user.id
            message_ids = kwargs["message_ids"] == (42, 43)
            user_id = kwargs["chat_id"] == "test_forwards"
            return from_chat_id and message_ids and user_id

        assert check_shortcut_signature(
            user.forward_messages_to, Bot.forward_messages, ["from_chat_id"], []
        )
        assert await check_shortcut_call(
            user.forward_messages_to, user.get_bot(), "forward_messages"
        )
        assert await check_defaults_handling(user.forward_messages_to, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "forward_messages", make_assertion)
        assert await user.forward_messages_to(chat_id="test_forwards", message_ids=(42, 43))

    async def test_instance_method_refund_star_payment(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["user_id"] == user.id and kwargs["telegram_payment_charge_id"] == 42

        assert check_shortcut_signature(
            user.refund_star_payment, Bot.refund_star_payment, ["user_id"], []
        )
        assert await check_shortcut_call(
            user.refund_star_payment, user.get_bot(), "refund_star_payment"
        )
        assert await check_defaults_handling(user.refund_star_payment, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "refund_star_payment", make_assertion)
        assert await user.refund_star_payment(telegram_payment_charge_id=42)

    async def test_instance_method_send_gift(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return (
                kwargs["user_id"] == user.id
                and kwargs["gift_id"] == "gift_id"
                and kwargs["text"] == "text"
                and kwargs["text_parse_mode"] == "text_parse_mode"
                and kwargs["text_entities"] == "text_entities"
            )

        assert check_shortcut_signature(user.send_gift, Bot.send_gift, ["user_id", "chat_id"], [])
        assert await check_shortcut_call(
            user.send_gift, user.get_bot(), "send_gift", ["chat_id", "user_id"]
        )
        assert await check_defaults_handling(user.send_gift, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "send_gift", make_assertion)
        assert await user.send_gift(
            gift_id="gift_id",
            text="text",
            text_parse_mode="text_parse_mode",
            text_entities="text_entities",
        )

    async def test_instance_method_gift_premium_subscription(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return (
                kwargs["user_id"] == user.id
                and kwargs["month_count"] == 3
                and kwargs["star_count"] == 1000
                and kwargs["text"] == "text"
                and kwargs["text_parse_mode"] == "text_parse_mode"
                and kwargs["text_entities"] == "text_entities"
            )

        assert check_shortcut_signature(
            user.gift_premium_subscription, Bot.gift_premium_subscription, ["user_id"], []
        )
        assert await check_shortcut_call(
            user.gift_premium_subscription,
            user.get_bot(),
            "gift_premium_subscription",
        )
        assert await check_defaults_handling(user.gift_premium_subscription, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "gift_premium_subscription", make_assertion)
        assert await user.gift_premium_subscription(
            month_count=3,
            star_count=1000,
            text="text",
            text_parse_mode="text_parse_mode",
            text_entities="text_entities",
        )

    async def test_instance_method_verify_user(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return (
                kwargs["user_id"] == user.id
                and kwargs["custom_description"] == "This is a custom description"
            )

        assert check_shortcut_signature(user.verify, Bot.verify_user, ["user_id"], [])
        assert await check_shortcut_call(user.verify, user.get_bot(), "verify_user")
        assert await check_defaults_handling(user.verify, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "verify_user", make_assertion)
        assert await user.verify(
            custom_description="This is a custom description",
        )

    async def test_instance_method_remove_user_verification(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["user_id"] == user.id

        assert check_shortcut_signature(
            user.remove_verification, Bot.remove_user_verification, ["user_id"], []
        )
        assert await check_shortcut_call(
            user.remove_verification, user.get_bot(), "remove_user_verification"
        )
        assert await check_defaults_handling(user.remove_verification, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "remove_user_verification", make_assertion)
        assert await user.remove_verification()

    async def test_instance_method_repost_story(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["from_chat_id"] == user.id

        assert check_shortcut_signature(
            User.repost_story,
            Bot.repost_story,
            [
                "from_chat_id",
            ],
            additional_kwargs=[],
        )
        assert await check_shortcut_call(
            user.repost_story,
            user.get_bot(),
            "repost_story",
            shortcut_kwargs=["from_chat_id"],
        )
        assert await check_defaults_handling(user.repost_story, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "repost_story", make_assertion)
        assert await user.repost_story(
            business_connection_id="bcid",
            from_story_id=123,
            active_period=3600,
        )

    async def test_instance_method_get_gifts(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return kwargs["user_id"] == user.id

        assert check_shortcut_signature(user.get_gifts, Bot.get_user_gifts, ["user_id"], [])
        assert await check_shortcut_call(user.get_gifts, user.get_bot(), "get_user_gifts")
        assert await check_defaults_handling(user.get_gifts, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "get_user_gifts", make_assertion)
        assert await user.get_gifts()

    async def test_instance_method_set_chat_member_tag(self, monkeypatch, user):
        async def make_assertion(*_, **kwargs):
            return (
                kwargs["user_id"] == user.id
                and kwargs["chat_id"] == "chat_id"
                and kwargs["tag"] == "tag"
            )

        assert check_shortcut_signature(
            user.set_chat_member_tag, Bot.set_chat_member_tag, ["user_id"], []
        )
        assert await check_shortcut_call(
            user.set_chat_member_tag, user.get_bot(), "set_chat_member_tag"
        )
        assert await check_defaults_handling(user.set_chat_member_tag, user.get_bot())

        monkeypatch.setattr(user.get_bot(), "set_chat_member_tag", make_assertion)
        assert await user.set_chat_member_tag(chat_id="chat_id", tag="tag")



#!/usr/bin/env python
#
# A library that provides a Python interface to the Telegram Bot API
# Copyright (C) 2015-2026
# Leandro Toledo de Souza <devs@python-telegram-bot.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser Public License for more details.
#
# You should have received a copy of the GNU Lesser Public License
# along with this program.  If not, see [http://www.gnu.org/licenses/].
"""Common base class for media objects"""

from typing import TYPE_CHECKING

from telegram._telegramobject import TelegramObject
from telegram._utils.defaultvalue import DEFAULT_NONE
from telegram._utils.types import JSONDict, ODVInput

if TYPE_CHECKING:
    from telegram import File


class _BaseMedium(TelegramObject):
    """Base class for objects representing the various media file types.
    Objects of this class are comparable in terms of equality. Two objects of this class are
    considered equal, if their :attr:`file_unique_id` is equal.

    Args:
        file_id (:obj:`str`): Identifier for this file, which can be used to download
            or reuse the file.
        file_unique_id (:obj:`str`): Unique identifier for this file, which
            is supposed to be the same over time and for different bots.
            Can't be used to download or reuse the file.
        file_size (:obj:`int`, optional): File size.

    Attributes:
        file_id (:obj:`str`): File identifier.
        file_unique_id (:obj:`str`): Unique identifier for this file, which
            is supposed to be the same over time and for different bots.
            Can't be used to download or reuse the file.
        file_size (:obj:`int`): Optional. File size.


    """

    __slots__ = ("file_id", "file_size", "file_unique_id")

    def __init__(
        self,
        file_id: str,
        file_unique_id: str,
        file_size: int | None = None,
        *,
        api_kwargs: JSONDict | None = None,
    ):
        super().__init__(api_kwargs=api_kwargs)

        # Required
        self.file_id: str = str(file_id)
        self.file_unique_id: str = str(file_unique_id)
        # Optionals
        self.file_size: int | None = file_size

        self._id_attrs = (self.file_unique_id,)

    async def get_file(
        self,
        *,
        read_timeout: ODVInput[float] = DEFAULT_NONE,
        write_timeout: ODVInput[float] = DEFAULT_NONE,
        connect_timeout: ODVInput[float] = DEFAULT_NONE,
        pool_timeout: ODVInput[float] = DEFAULT_NONE,
        api_kwargs: JSONDict | None = None,
    ) -> "File":
        """Convenience wrapper over :meth:`telegram.Bot.get_file`

        For the documentation of the arguments, please see :meth:`telegram.Bot.get_file`.

        Returns:
            :class:`telegram.File`

        Raises:
            :class:`telegram.error.TelegramError`

        """
        return await self.get_bot().get_file(
            file_id=self.file_id,
            read_timeout=read_timeout,
            write_timeout=write_timeout,
            connect_timeout=connect_timeout,
            pool_timeout=pool_timeout,
            api_kwargs=api_kwargs,
        )
